#!/bin/bash

set -e

migration_compose_file=./docker-compose-migration.yaml
migration_env_file=./.env-migration

function wait_for_log_msg() {
    local service_name=$1
    local log_msg=$2
    local wait_msg=$3

    while ! docker-compose -f $migration_compose_file --env-file $migration_env_file logs "$service_name" 2>&1 | grep -q "$log_msg";
    do
        sleep 5
        log "$wait_msg"
    done
}

function print_ssl_warning() {
    deny_file=$1
    log "Permission denied on file/directory $deny_file !"
    cat <<EOF
    Please make sure about good permissions of these files/dirs:
      - ssl : it should be writable by anyone (777, or drwxrwxrwx)
    These files are optionals:
      - ssl/certificate.crt : it should be readable by anyone (666, or -r--r--r--)
      - ssl/private.key : it should be readable by anyone (666, or -r--r--r--)
      - ssl/dhparam.pem : it should be readable by anyone (666, or -r--r--r--)

EOF
    log "Permission problems, see the message above, exiting..."
    exit 1
}

function check_ssl_dir() {
    local ssl_path="./ssl"
    if [ -d "$ssl_path" ];then
        if (touch "$ssl_path/.test_permission");then
            log "$ssl_path is writable: OK"
            rm "$ssl_path/.test_permission"
        else
            print_ssl_warning "ssl/"
        fi

        for test_file in $(find "$ssl_path" -maxdepth 1 -type f);do
            if (cat $test_file > /dev/null);then
                log "$(basename $test_file) is readable: OK"
            else
                print_ssl_warning "$(basename $test_file)"
            fi
        done
    fi
}

function log() {
    local msg=$1

    echo "[MIGRATION_ORCHESTRATOR] $msg"
}

function on_error(){
    log "Some error occured in: $(caller)"

    # rollback

    reset_migration_lock
}

function clean_up() {
    reset_migration_lock
}

function reset_migration_lock() {
    # Reset the value of the MIGRATION_LOCK variable
    sed -i 's/MIGRATION_LOCK.*/MIGRATION_LOCK=locked/g' .env-migration
    log "Migration lock reset"
}

function rollback() {
    log "Starting rollback process..." 

    # Set the value of the ROLLBACK variable to true
    sed -i 's/ROLLBACK.*/ROLLBACK=true/g' .env-migration

    # Reset the value of the ROLLBACK variable
    sed -i 's/ROLLBACK.*/ROLLBACK=false/g' .env-migration

    log "Rollback process finished." 
}

trap 'on_error' ERR INT

# Check the permissions of the './ssl' dir
check_ssl_dir

# Open the MIGRATION_LOCK so the entrypoints are enabled in the migration images
sed -i 's/MIGRATION_LOCK.*/MIGRATION_LOCK=open/g' .env-migration


###################
### MIGRATION STEPS
###################

# Check if customer files are in the current directory
migration_curr_dir_err_msg="Migration should be started in the same directory where the current docker-compose stack is defined."
if [ ! -f .env ];
then
    log "Could not find '.env' file. $migration_curr_dir_err_msg"
    log "Exiting..."
    reset_migration_lock
    exit 1
fi

if [ ! -f docker-compose.yml ];
then
    log "Could not find 'docker-compose.yml' file. $migration_curr_dir_err_msg"
    log "Exiting..."
    reset_migration_lock
    exit 1
fi

# Create backup from customer .env and docker-compose.yml files (9.10.11-gen2)
log "Create backup from customer '.env' and 'docker-compose.yml' files"
cp ./.env .env-bak
cp ./docker-compose.yml docker-compose.yml.bak


log "Comparing current customer files to stock 9.10.11-gen2"

customer_current_env_file=./.env
customer_current_compose_file=./docker-compose.yml
default_env_file=./.env-9.10.11-gen2
default_compose_file=docker-compose-9.10.11-gen2.yml

# Check if diff is on the path
if ! command -v diff > /dev/null;
then
    log "'diff' command is not available"
    log "Exiting..."
    exit 1
fi

log "Comparing customer '.env' file to stock 9.10.11-gen2"
log "Please review this differences manually and adapt the AI HUB X configuration accordingly."
set +e
diff $customer_current_env_file $default_env_file || true
set -e
log ""

log "Comparing customer 'docker-compose.yml' file to stock 9.10.11-gen2"
log "Please review this differences manually and adapt the AI HUB X configuration accordingly."
set +e
diff $customer_current_compose_file $default_compose_file || true
set -e
log ""


# Rename current customer files to prepare for further steps
log "Renaming current customer files to prepare for further steps..."
# .env --> .env-customer
mv $customer_current_env_file .env-customer && customer_current_env_file=.env-customer
# docker-compose.yml --> docker-compose-customer.yml
mv $customer_current_compose_file docker-compose-customer.yml && customer_current_compose_file=docker-compose-customer.yml

# Change the .env mount in docker-compose-customer.yml to .env-customer
sed -i "s,./.env:,./$customer_current_env_file:,g" $customer_current_compose_file
# Change the docker-compose.yml mount in docker-compose-customer.yml to docker-compose-customer.yml
sed -i "s,./docker-compose.yml:,./$customer_current_compose_file:,g" $customer_current_compose_file

log "Starting migration..."

## Python script to migrate .env file (create backup)
log "Starting .env file migration..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up --exit-code-from env-migrator env-migrator
log ".env migration finished."

# Rename the result of env migration and the new AI HUB X compose file
log "Renaming the result of ENV_MIGRATOR..."
# .env-aix --> .env
mv .env-aix .env
# docker-compose-aix.yml --> docker-compose.yml
mv docker-compose-aix.yml docker-compose.yml

# Configure kcadm so later migrator services can access the new Keycloak
log "Starting kcadm configurer..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up --exit-code-from kcadm-configurer kcadm-configurer
log "kcadm configurer finished."

# Migrate KC DB
log "Starting Keycloak database migration..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up --exit-code-from kc-db-migrator kc-db-migrator
log "Keycloak database migration finished."

# Run deployment-init (with new KC)
log "Starting deployment-init..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up --exit-code-from deployment-init deployment-init
log "deployment-init finished."

# Copy value of JOBAGENT_SSO_CLIENT_SECRET from .env to .env-migration as Jobagent needs it to start up
# JOBAGENT_SSO_CLIENT_SECRET is a new key, it cannot be migrated from the customer config
# this extra step is needed as the value of JOBAGENT_SSO_CLIENT_SECRET only gets populated by deployment-init
log "Copying JOBAGENT_SSO_CLIENT_SECRET into .env-migration..."
JOBAGENT_SSO_CLIENT_SECRET=$(grep '^JOBAGENT_SSO_CLIENT_SECRET' .env|cut -f2 -d'=')
sed -i "s,JOBAGENT_SSO_CLIENT_SECRET=.*,JOBAGENT_SSO_CLIENT_SECRET=$JOBAGENT_SSO_CLIENT_SECRET,g" .env-migration

# Migrate KC
log "Starting Keycloak migration..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up --exit-code-from kc-migrator kc-migrator
log "Keycloak migration finished."

# STOP KC and PG used with kc-migrator
log "Stopping migration helper services (KC and PG)..."
docker-compose -f $migration_compose_file --env-file $migration_env_file down
log "Helper services stopped."

# Migrate AI HUB
log ""
log "Starting AI HUB migration..."
log ""

# AI HUB Backend
log "Starting AI HUB backend migration"
docker-compose -f $migration_compose_file --env-file $migration_env_file up -d aihub-backend

aihub_backend_migration_msg="Migration successful"
aihub_backend_startup_msg="Scheduler was started successfully"

wait_for_log_msg aihub-backend "$aihub_backend_migration_msg" "Waiting for AI HUB backend migration to finish..."

wait_for_log_msg aihub-backend "$aihub_backend_startup_msg" "Waiting for AI HUB Backend to start up..."

log "AI HUB backend migration finished."

# Jobagent
log "Starting Jobagent migration..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up -d aihub-job-agent

job_agent_migration_msg="Migration successful"
job_agent_startup_msg="Starting to accept jobs"

wait_for_log_msg aihub-job-agent "$job_agent_migration_msg" "Waiting for Jobagent migration to finish..."

wait_for_log_msg aihub-job-agent "$job_agent_startup_msg" "Waiting for Jobagent to start up..."

log "Jobagent migration finished."

# Scoring Agent
log "Starting Scoring Agent migration..."

docker-compose -f $migration_compose_file --env-file $migration_env_file up -d scoring-agent

scoring_agent_migration_msg="Migration successful"
scoring_agent_startup_msg="Started Application"

wait_for_log_msg scoring-agent "$scoring_agent_migration_msg" "Waiting for Scoring Agent migration to finish..."

wait_for_log_msg scoring-agent "$scoring_agent_startup_msg" "Waiting for Scoring Agent to start up..."

log "Scoring Agent migration finished."

log "AI HUB migration finished."

# Migrate GF
log "Starting Grafana migration..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up --exit-code-from gf-migrator gf-migrator
log "Grafana migration finished."

# Migrate PA licences
log "Starting Platform Admin licences migration..."
docker-compose -f $migration_compose_file --env-file $migration_env_file up --exit-code-from platform-admin-migrator platform-admin-migrator
log "Platform Admin licences migration finished."

log "Stop all services"
docker-compose -f $migration_compose_file --env-file $migration_env_file down

###################
### Clean up
###################

clean_up

log "Migration finished."
